//
// © 2014, 2022 David Vines
//
// Creative Commons License
//
// This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
//
// Big 'C' Character is used to namespace the functions
//
// little 'c' character contains the character being created.

/* globals Character: true, character: true, options, Rules */
Character.updateEndDate = function(cid,p,delta) {
    return Character.returnFalse(function() {
        character[cid].periods[p].endmonth += delta;
        character[cid].periods[p].endmonthtext = character[cid].genre.dateMonthAsText(character[cid].periods[p].endmonth+character[cid].birthmonth);
        if (character[cid].periods[p+1]) {
            // Update start of next period
            character[cid].periods[p+1].startmonth = character[cid].periods[p].endmonth+1;
            character[cid].periods[p+1].startmonthtext = character[cid].genre.dateMonthAsText(character[cid].periods[p+1].startmonth+character[cid].birthmonth);
            // Future period now at zero or negative length?, if so we need to bump its end month too
            if (character[cid].periods[p+1].startmonth > character[cid].periods[p+1].endmonth) {
                    // By just enough to make it one month in length
                    Character.updateEndDate(cid, p+1, character[cid].periods[p+1].startmonth - character[cid].periods[p+1].endmonth + 1);
            }
        }
        Character.incrementVersion(cid);
        Character.populatePeriods(cid,true);
        Character.populateSkills(cid,true);
    });
};

Character.copyPreviousPeriodSkills = function(cid,p) {
    return Character.returnFalse(function() {
        if (character[cid].periods[p].iptotal < character[cid].interestPoints) {
            const prev = p-1;
            const table = document.getElementById("c"+cid+"-period"+p+"-details").firstChild.nextSibling;
            for(let i=0; i<character[cid].periods[prev].interests.length; i++) {
                const skillname = character[cid].periods[prev].interests[i].skill;
                var missing = true;
                for(let j=0; j<character[cid].periods[p].interests.length; j++) {
                    if (character[cid].periods[p].interests[j].skill === skillname) {
                        missing = false; break;
                    }
                }
                let ips = character[cid].periods[prev].interests[i].ip;
                if (ips > (character[cid].interestPoints-character[cid].periods[p].iptotal)) ips = character[cid].interestPoints-character[cid].periods[p].iptotal;
                if (missing) {
                    Character.addInterestKnowingPeriod(cid,p,table,skillname,ips,true);
                }
                if (character[cid].periods[p].iptotal == character[cid].interestPoints) break;
            }
            Character.incrementVersion(cid);
            Character.populatePeriods(cid,true);
            Character.populateSkills(cid,true);
        }
    });
};

Character.updateFluency = function(cid,p,fc) {
    return Character.returnFalse(function() {
        character[cid].periods[p].childhoodLanguage = fc[fc.selectedIndex].text;
        Character.incrementVersion(cid);
        Character.populateSkills(cid,true);
    });
};

Character.updatePoints = function(cid,p,i,delta) {
    return Character.returnFalse(function() {
        character[cid].periods[p].interests[i].ip += delta;
        character[cid].periods[p].iptotal += delta;
        Character.updateSkillRow(cid,p,i);
        Character.updateSkillRowButtons(cid,p);
        Character.updateStatus(cid,p);
        Character.incrementVersion(cid);
        Character.populateSkills(cid,true);
    });
};

Character.updateTeacher = function(cid,p,i,sc) {
    return Character.returnFalse(function() {
        character[cid].periods[p].interests[i].teacher = sc.checked;
        Character.incrementVersion(cid);
        Character.populateSkills(cid,true);
    });
};

Character.updateSkillRowButtons = function(cid,p) {
    for(var i=0; i<character[cid].periods[p].interests.length; i++) {
        var interestup = document.getElementById("c"+cid+"-period"+p+"-interest"+i+"-ipup");
        var interestdown = document.getElementById("c"+cid+"-period"+p+"-interest"+i+"-ipdown");
        interestup.disabled = (character[cid].periods[p].iptotal >= character[cid].interestPoints);
        interestdown.disabled = (character[cid].periods[p].interests[i].ip < 1);
    }
};

Character.deleteInterest = function(cid,p,interest) {
    return Character.returnFalse(function() {
        character[cid].periods[p].iptotal -= character[cid].periods[p].interests[interest].ip;
        character[cid].periods[p].interests.splice(interest,1);
        for(var i=interest; i<character[cid].periods[p].interests.length; i++) {
            Character.updateSkillRow(cid,p,i);
        }
        var row = document.getElementById("c"+cid+"-period"+p+"-interest"+character[cid].periods[p].interests.length);
        Character.deleteElement(cid,row);
        Character.updateSkillRowButtons(cid,p);
        Character.updateStatus(cid,p);
        Character.incrementVersion(cid);
        Character.populateSkills(cid,true);
    });
};

Character.populatePeriodTitle = function(cid,p,li) {
    return Character.returnFalse(function() {
        var newtitle = document.getElementById("c"+cid+"-period"+p+"-titletextarea").value;
        character[cid].periods[p].title = newtitle;
        Character.setIdText(cid,"sheetnav"+li,newtitle);
        document.getElementById("c"+cid+"-sheetnav"+li).title = newtitle;
        Character.incrementVersion(cid);
        Character.populateJSON(cid,true);
    });
};

Character.populatePeriodDescription = function(cid,p) {
    return Character.returnFalse(function() {
        character[cid].periods[p].description =  document.getElementById("c"+cid+"-period"+p+"-descriptiontextarea").value;
        Character.incrementVersion(cid);
        Character.populateJSON(cid,true);
    });
};

Character.birthdaydelta = function(cid,delta) {
    return Character.returnFalse(function() {
        character[cid].birthtime += delta;
        character[cid].birthmonth = character[cid].genre.asMonth(character[cid].birthtime);
        character[cid].birthmonthtext = character[cid].genre.dateMonthAsText(character[cid].birthmonth);
        character[cid].birthday = character[cid].genre.dayMonthYearAsText(character[cid].birthtime,character[cid].birthmonth);
        character[cid].periods[character[cid].periods.length-1].endmonth = character[cid].gamemonth - character[cid].birthmonth;
        for(var i=0; i<character[cid].periods.length; i++) {
            character[cid].periods[i].startmonthtext = character[cid].genre.dateMonthAsText(character[cid].periods[i].startmonth+character[cid].birthmonth);
            character[cid].periods[i].endmonthtext = character[cid].genre.dateMonthAsText(character[cid].periods[i].endmonth+character[cid].birthmonth);
        }
        Character.incrementVersion(cid);
        Character.populateAll(cid,true); // all the dates will have shifted in sympathy
    });
} ;

Character.gamedaydelta = function(cid,delta) {
    return Character.returnFalse(function() {
        character[cid].gametime += delta;
        character[cid].gamemonth = character[cid].genre.asMonth(character[cid].gametime);
        character[cid].gamemonthtext = character[cid].genre.dateMonthAsText(character[cid].gamemonth);
        character[cid].gameday = character[cid].genre.dayMonthYearAsText(character[cid].gametime,character[cid].gamemonth);
        character[cid].periods[character[cid].periods.length-1].endmonth = character[cid].gamemonth - character[cid].birthmonth;
        character[cid].periods[character[cid].periods.length-1].endmonthtext = character[cid].gamemonthtext;
        Character.incrementVersion(cid);
        Character.populateAll(cid,true); // The last period will have changed length (possibly to a negative length)
    });
} ;

Character.updateDerivedStats = function(cid) {
    character[cid].morale = Rules.startingMorale(character[cid]);
    character[cid].woundPoints = Rules.woundPoints(character[cid]);
    character[cid].interestPoints = Rules.interestPoints(character[cid]);
    for(var p=0; p<character[cid].periods.length; p++) {
        character[cid].periods[p].iptotal = 0;
        for(var i=0; i<character[cid].periods[p].interests.length; i++) {
            character[cid].periods[p].iptotal += character[cid].periods[p].interests[i].ip;
        }
        Character.updateStatus(cid,p);
    }
    Character.populateAll(cid,true);
};

Character.statSwap = function(cid,stat,sc) {
    return  Character.returnFalse(function() {
        var option = sc[sc.selectedIndex].text;
        if (option === "") {
            return;
        }
        var shorthand = { "Strength": "str",
                          "Dexterity": "dex",
                          "Agility": "agi",
                          "Constitution": "con",
                          "Appearance": "app",
                          "Intellect": "int",
                          "Wit": "wit",
                          "Will": "will"
                        };
        var that = shorthand[option];
        var other = character[cid][that];
        character[cid][that] = character[cid][stat];
        character[cid][stat] = other;
        Character.updateDerivedStats(cid);
        Character.incrementVersion(cid);
        sc.selectedIndex = 0;
    });
};

Character.init = function(cid) {
    // Copy across the form fields to the ones actually sent back to WordPress (in the hidden div)
    Character.setTextArea(cid,"djv_nametextarea",character[cid].name);
    Character.setTextArea(cid,"djv_genre",character[cid].genre.toString());
    Character.setTextArea(cid,"djv_playernametextarea",character[cid].playerName);

    // Populate the skill list div with buttons
    const skilllist = document.getElementById("c"+cid+"-skilllist").parentNode;
    const childhoodfluency = document.getElementById("c"+cid+"-period@@-childhoodfluencylanguage");
    const addToSkillList = function(skilllist,category,dom) {
        const svgNS = "http://www.w3.org/2000/svg";
        const DOM_svg_expand = document.createElementNS(svgNS, "svg");
        DOM_svg_expand.setAttribute("viewBox","0 0 10 10");
        DOM_svg_expand.setAttribute("width","12");
        DOM_svg_expand.setAttribute("height","12");
        DOM_svg_expand.setAttribute("fill","darkcyan");
        DOM_svg_expand.setAttribute("stroke","black");
        DOM_svg_expand.setAttribute("class","");
        const triangleRight = document.createElementNS(svgNS,"polygon");
        triangleRight.setAttribute("points","0,10 0,0 10,5");
        DOM_svg_expand.appendChild(triangleRight);
        const DOM_svg_collapse = document.createElementNS(svgNS,"svg");
        DOM_svg_collapse.setAttribute("viewBox","0 0 10 10");
        DOM_svg_collapse.setAttribute("width","12");
        DOM_svg_collapse.setAttribute("height","12");
        DOM_svg_collapse.setAttribute("fill","darkcyan");
        DOM_svg_collapse.setAttribute("stroke","black");
        DOM_svg_collapse.setAttribute("class","hidden");
        const triangleDown = document.createElementNS(svgNS,"polygon");
        triangleDown.setAttribute("points","0,0 10,0 5,10");
        DOM_svg_collapse.appendChild(triangleDown);
        const DOM_button = document.createElement("button");
        DOM_button.appendChild(DOM_svg_expand);
        DOM_button.appendChild(DOM_svg_collapse);
        DOM_button.appendChild(document.createTextNode(category));
        const DOM_h4 = document.createElement("h4");
        DOM_h4.appendChild(DOM_button);
        const DOM_div = document.createElement("div");
        DOM_div.id = "c"+cid+"-skills-category-"+encodeURI(category); // to encode the blanks
        DOM_div.className = "skillsection";
        DOM_div.appendChild(DOM_h4);
        dom.className = "hidden";
        DOM_div.appendChild(dom);
        skilllist.appendChild(DOM_div);
        // Now the onclick action for the button
        DOM_button.onclick = Character.returnFalse(function() {
            const oldClass = dom.className;
            const newClass = (oldClass ? "" : "hidden");
            DOM_svg_expand.setAttribute("class",oldClass);
            DOM_svg_collapse.setAttribute("class",newClass);
            dom.className = newClass;
        });
    };
    var DOM_h4, DOM_form, DOM_table, DOM_tbody, DOM_tr, DOM_td, DOM_button;
    for(var category in Rules.skilllist) {
        if (Rules.skilllist.hasOwnProperty(category)) {
            DOM_table = document.createElement("table");
            DOM_table.style.width = "100%";
            DOM_table.style.tableLayout = "fixed";
            DOM_tbody = document.createElement("tbody");
            DOM_tr = undefined;
            for(var skill in Rules.skilllist[category]) {
                if (Rules.skilllist[category].hasOwnProperty(skill)) {
                    DOM_td = document.createElement("td");
                    DOM_td.appendChild(Character.createButton(skill,Character.addInterest(cid,skill,true)));
                    // Flip-flop the cells into rows so that we get two columns
                    if (DOM_tr) {
                        DOM_tr.appendChild(DOM_td);
                        DOM_tbody.appendChild(DOM_tr);
                        DOM_tr = undefined;
                    } else {
                        DOM_tr = document.createElement("tr");
                        DOM_tr.appendChild(DOM_td);
                    }
                    if (skill.indexOf("Fluency(") === 0) {
                        var dialect = skill.slice(8,-1);
                        let DOM_option = new Option(dialect,dialect,true,false);
                        childhoodfluency.appendChild(DOM_option);
                    }
                }
            }
            // one cell row at the end?
            if (DOM_tr) {
                DOM_td = document.createElement("td");
                DOM_tr.appendChild(DOM_td);
                DOM_tbody.appendChild(DOM_tr);
            }
            DOM_table.appendChild(DOM_tbody);

            addToSkillList(skilllist,category,DOM_table);
        }
    }

    // Other language
    const DOM_subdiv = document.createElement("div");
    DOM_h4 = document.createElement("h4");
    DOM_h4.appendChild(document.createTextNode("Add unlisted Language"));
    DOM_subdiv.appendChild(DOM_h4);
    DOM_form = document.createElement("form");
    DOM_table = document.createElement("table");
    DOM_table.style.width = "100%";
    DOM_table.style.tableLayout = "fixed";
    DOM_tbody = document.createElement("tbody");
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 3;
    DOM_td.appendChild(Character.createTextArea(cid,"skill-add-language",1,20));
    DOM_tr.appendChild(DOM_td);
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 1;
    DOM_button = Character.createButton("Add",Character.returnFalse(function(){
        var name = document.getElementById("c"+cid+"-skill-add-language").value;
        Character.addOtherSkill(cid,"Fluency("+name+")",["int"],true);
        Character.addOtherSkill(cid,"Literacy("+name+")",["int"],true);
    }));
    DOM_td.appendChild(DOM_button);
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    DOM_table.appendChild(DOM_tbody);
    DOM_form.appendChild(DOM_table);
    DOM_subdiv.appendChild(DOM_form);

    // Other Religion
    DOM_h4 = document.createElement("h4");
    DOM_h4.appendChild(document.createTextNode("Add unlisted Religion"));
    DOM_subdiv.appendChild(DOM_h4);
    DOM_form = document.createElement("form");
    DOM_table = document.createElement("table");
    DOM_table.style.width = "100%";
    DOM_table.style.tableLayout = "fixed";
    DOM_tbody = document.createElement("tbody");
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 3;
    DOM_td.appendChild(Character.createTextArea(cid,"skill-add-religion",1,20));
    DOM_tr.appendChild(DOM_td);
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 1;
    DOM_button = Character.createButton("Add",Character.returnFalse(function(){
        var name = document.getElementById("c"+cid+"-skill-add-religion").value;
        Character.addOtherSkill(cid,"Ritual("+name+")",["will"],true);
        Character.addOtherSkill(cid,"Theology("+name+")",["int"],true);
    }));
    DOM_td.appendChild(DOM_button);
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    DOM_table.appendChild(DOM_tbody);
    DOM_form.appendChild(DOM_table);
    DOM_subdiv.appendChild(DOM_form);

    // Other Ritual
    DOM_h4 = document.createElement("h4");
    DOM_h4.appendChild(document.createTextNode("Add unlisted Ritual"));
    DOM_subdiv.appendChild(DOM_h4);
    DOM_form = document.createElement("form");
    DOM_table = document.createElement("table");
    DOM_table.style.width = "100%";
    DOM_table.style.tableLayout = "fixed";
    DOM_tbody = document.createElement("tbody");
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 3;
    DOM_td.appendChild(Character.createTextArea(cid,"skill-add-ritual",1,20));
    DOM_tr.appendChild(DOM_td);
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 1;
    DOM_button = Character.createButton("Add",Character.returnFalse(function(){
        var name = document.getElementById("c"+cid+"-skill-add-ritual").value;
        Character.addOtherSkill(cid,"Ritual("+name+")",["will"]);
    }));
    DOM_td.appendChild(DOM_button);
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    DOM_table.appendChild(DOM_tbody);
    DOM_form.appendChild(DOM_table);
    DOM_subdiv.appendChild(DOM_form);

    // Other Intrigue
    DOM_h4 = document.createElement("h4");
    DOM_h4.appendChild(document.createTextNode("Add unlisted Intrigue"));
    DOM_subdiv.appendChild(DOM_h4);
    DOM_form = document.createElement("form");
    DOM_table = document.createElement("table");
    DOM_table.style.width = "100%";
    DOM_table.style.tableLayout = "fixed";
    DOM_tbody = document.createElement("tbody");
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 3;
    DOM_td.appendChild(Character.createTextArea(cid,"skill-add-intrigue",1,20));
    DOM_tr.appendChild(DOM_td);
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 1;
    DOM_button = Character.createButton("Add",Character.returnFalse(function(){
        var name = document.getElementById("c"+cid+"-skill-add-intrigue").value;
        Character.addOtherSkill(cid,"Intrigue("+name+")",["wit"],true);
    }));
    DOM_td.appendChild(DOM_button);
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    DOM_table.appendChild(DOM_tbody);
    DOM_form.appendChild(DOM_table);
    DOM_subdiv.appendChild(DOM_form);

    // Other Art/Craft
    DOM_h4 = document.createElement("h4");
    DOM_h4.appendChild(document.createTextNode("Add unlisted Art/Craft"));
    DOM_subdiv.appendChild(DOM_h4);
    DOM_form = document.createElement("form");
    DOM_table = document.createElement("table");
    DOM_table.style.width = "100%";
    DOM_table.style.tableLayout = "fixed";
    DOM_tbody = document.createElement("tbody");
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 3;
    DOM_td.appendChild(Character.createTextArea(cid,"skill-add-artcraft",1,20));
    DOM_tr.appendChild(DOM_td);
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 1;
    DOM_button = Character.createButton("Add",Character.returnFalse(function(){
        var name = document.getElementById("c"+cid+"-skill-add-artcraft").value;
        Character.addOtherSkill(cid,"Art/Craft("+name+")",["dex"],true);
    }));
    DOM_td.appendChild(DOM_button);
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    DOM_table.appendChild(DOM_tbody);
    DOM_form.appendChild(DOM_table);
    DOM_subdiv.appendChild(DOM_form);

    // Other Sport
    DOM_h4 = document.createElement("h4");
    DOM_h4.appendChild(document.createTextNode("Add unlisted Sport"));
    DOM_subdiv.appendChild(DOM_h4);
    DOM_form = document.createElement("form");
    DOM_table = document.createElement("table");
    DOM_table.style.width = "100%";
    DOM_table.style.tableLayout = "fixed";
    DOM_tbody = document.createElement("tbody");
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 3;
    DOM_td.appendChild(Character.createTextArea(cid,"skill-add-sport",1,20));
    DOM_tr.appendChild(DOM_td);
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 1;
    DOM_button = Character.createButton("Add",Character.returnFalse(function(){
        var name = document.getElementById("c"+cid+"-skill-add-sport").value;
        Character.addOtherSkill(cid,"Sport("+name+")",["agi"],true);
    }));
    DOM_td.appendChild(DOM_button);
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    DOM_table.appendChild(DOM_tbody);
    DOM_form.appendChild(DOM_table);
    DOM_subdiv.appendChild(DOM_form);

    // The 'other' skills
    DOM_h4 = document.createElement("h4");
    DOM_h4.appendChild(document.createTextNode("Add Unlisted Other Skill"));
    DOM_subdiv.appendChild(DOM_h4);
    DOM_form = document.createElement("form");
    DOM_table = document.createElement("table");
    DOM_table.style.width = "100%";
    DOM_table.style.tableLayout = "fixed";
    DOM_tbody = document.createElement("tbody");
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 4;
    DOM_td.appendChild(Character.createTextArea(cid,"skill-add-name",1,24));
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 4;
    DOM_td.appendChild(document.createTextNode("Attributes"));
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    var addStats = function(tbody,stats) {
        var DOM_tr = document.createElement("tr");
        for (var i=0; i<stats.length; i++) {
            var DOM_td = document.createElement("td");
            DOM_td.className = "right-align";
            DOM_td.appendChild(document.createTextNode(stats[i]+" "));
            var DOM_checkbox = document.createElement("input");
            DOM_checkbox.type = "checkbox";
            DOM_checkbox.id = "c"+cid+"-skill-add-"+stats[i];
            DOM_checkbox.checked = false;
            DOM_td.appendChild(DOM_checkbox);
            DOM_tr.appendChild(DOM_td);
        }
        tbody.appendChild(DOM_tr);
    };
    addStats(DOM_tbody,Character.allstats.slice(0,4));
    addStats(DOM_tbody,Character.allstats.slice(4));
    DOM_tr = document.createElement("tr");
    DOM_td = document.createElement("td");
    DOM_td.colSpan = 4;
    DOM_button = Character.createButton("Add New Skill",Character.returnFalse(function(){
        var name = document.getElementById("c"+cid+"-skill-add-name").value;
        var stats = [];
        for(var i=0; i<Character.allstats.length; i++) {
            if (document.getElementById("c"+cid+"-skill-add-"+Character.allstats[i]).checked) {
                stats.push(Character.allstats[i]);
            }
        }
        Character.addOtherSkill(cid,name,stats,true);
    }));
    DOM_td.appendChild(DOM_button);
    DOM_tr.appendChild(DOM_td);
    DOM_tbody.appendChild(DOM_tr);
    DOM_table.appendChild(DOM_tbody);
    DOM_form.appendChild(DOM_table);
    DOM_subdiv.appendChild(DOM_form);
    addToSkillList(skilllist,"Add a skill to the 'Added Skills' section",DOM_subdiv);
};

if (typeof options !== "undefined") {
    for(var charindex = 0; charindex < options.ids.length; charindex++) {
        let ineditmode = Character.initPage(options.ids[charindex]); // Page first so that the new skills (if any) get loaded
        if (ineditmode) Character.init(options.ids[charindex]);
        Character.populateAll(options.ids[charindex],ineditmode);
    }

    const titleField = document.getElementById("title");
    const nameField = document.getElementById("c1-nametextarea"); // We only link the first character on the page (there will normally only be one)

    if (titleField && nameField) {
        const updater = function(e) {
            if (e.target === titleField) nameField.value = e.target.value;
            if (e.target === nameField) titleField.value = e.target.value;
        };
        titleField.addEventListener('input',updater);
        nameField.addEventListener('input',updater);
    }
}
